# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from poodle_async_full.models.mod_scorm_get_scorm_scoes_response_scoes_inner_extradata_inner import ModScormGetScormScoesResponseScoesInnerExtradataInner
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class ModScormGetScormScoesResponseScoesInner(BaseModel):
    """
    SCORM SCO data
    """ # noqa: E501
    extradata: Optional[List[ModScormGetScormScoesResponseScoesInnerExtradataInner]] = Field(default=None, description="Additional SCO data")
    id: Optional[StrictInt] = Field(description="sco id")
    identifier: Optional[StrictStr] = Field(description="identifier")
    launch: Optional[StrictStr] = Field(description="launch file")
    manifest: Optional[StrictStr] = Field(description="manifest id")
    organization: Optional[StrictStr] = Field(description="organization id")
    parent: Optional[StrictStr] = Field(description="parent")
    scorm: Optional[StrictInt] = Field(description="scorm id")
    scormtype: Optional[StrictStr] = Field(description="scorm type (asset, sco)")
    sortorder: Optional[StrictInt] = Field(description="sort order")
    title: Optional[StrictStr] = Field(description="sco title")
    __properties: ClassVar[List[str]] = ["extradata", "id", "identifier", "launch", "manifest", "organization", "parent", "scorm", "scormtype", "sortorder", "title"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ModScormGetScormScoesResponseScoesInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in extradata (list)
        _items = []
        if self.extradata:
            for _item_extradata in self.extradata:
                if _item_extradata:
                    _items.append(_item_extradata.to_dict())
            _dict['extradata'] = _items
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if identifier (nullable) is None
        # and model_fields_set contains the field
        if self.identifier is None and "identifier" in self.model_fields_set:
            _dict['identifier'] = None

        # set to None if launch (nullable) is None
        # and model_fields_set contains the field
        if self.launch is None and "launch" in self.model_fields_set:
            _dict['launch'] = None

        # set to None if manifest (nullable) is None
        # and model_fields_set contains the field
        if self.manifest is None and "manifest" in self.model_fields_set:
            _dict['manifest'] = None

        # set to None if organization (nullable) is None
        # and model_fields_set contains the field
        if self.organization is None and "organization" in self.model_fields_set:
            _dict['organization'] = None

        # set to None if parent (nullable) is None
        # and model_fields_set contains the field
        if self.parent is None and "parent" in self.model_fields_set:
            _dict['parent'] = None

        # set to None if scorm (nullable) is None
        # and model_fields_set contains the field
        if self.scorm is None and "scorm" in self.model_fields_set:
            _dict['scorm'] = None

        # set to None if scormtype (nullable) is None
        # and model_fields_set contains the field
        if self.scormtype is None and "scormtype" in self.model_fields_set:
            _dict['scormtype'] = None

        # set to None if sortorder (nullable) is None
        # and model_fields_set contains the field
        if self.sortorder is None and "sortorder" in self.model_fields_set:
            _dict['sortorder'] = None

        # set to None if title (nullable) is None
        # and model_fields_set contains the field
        if self.title is None and "title" in self.model_fields_set:
            _dict['title'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ModScormGetScormScoesResponseScoesInner from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "ModScormGetScormScoesResponseScoesInner" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "extradata": [ModScormGetScormScoesResponseScoesInnerExtradataInner.from_dict(_item) for _item in obj["extradata"]] if obj.get("extradata") is not None else None,
            "id": obj.get("id"),
            "identifier": obj.get("identifier"),
            "launch": obj.get("launch"),
            "manifest": obj.get("manifest"),
            "organization": obj.get("organization"),
            "parent": obj.get("parent"),
            "scorm": obj.get("scorm"),
            "scormtype": obj.get("scormtype"),
            "sortorder": obj.get("sortorder"),
            "title": obj.get("title")
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from poodle_async_full.models.mod_h5pactivity_get_results_response_attempts_inner_results_inner_options_inner import ModH5pactivityGetResultsResponseAttemptsInnerResultsInnerOptionsInner
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class ModH5pactivityGetResultsResponseAttemptsInnerResultsInner(BaseModel):
    """
    A single result statement tracking information
    """ # noqa: E501
    answerlabel: Optional[StrictStr] = Field(default=None, description="Label used for user answers")
    attemptid: Optional[StrictInt] = Field(description="ID of the H5P attempt")
    completion: Optional[StrictInt] = Field(default=None, description="Result completion")
    content: Optional[StrictStr] = Field(default=None, description="Result extra content")
    correctlabel: Optional[StrictStr] = Field(default=None, description="Label used for correct answers")
    description: Optional[StrictStr] = Field(description="Result description")
    duration: Optional[StrictInt] = Field(default=0, description="Result duration in seconds")
    id: Optional[StrictInt] = Field(description="ID of the context")
    interactiontype: Optional[StrictStr] = Field(description="Interaction type")
    maxscore: Optional[StrictInt] = Field(description="Result max score")
    options: Optional[List[ModH5pactivityGetResultsResponseAttemptsInnerResultsInnerOptionsInner]] = Field(default=None, description="The statement options")
    optionslabel: Optional[StrictStr] = Field(default=None, description="Label used for result options")
    rawscore: Optional[StrictInt] = Field(description="Result score value")
    subcontent: Optional[StrictStr] = Field(description="Subcontent identifier")
    success: Optional[StrictInt] = Field(default=None, description="Result success")
    timecreated: Optional[StrictInt] = Field(description="Result creation")
    track: Optional[StrictBool] = Field(default=None, description="If the result has valid track information")
    __properties: ClassVar[List[str]] = ["answerlabel", "attemptid", "completion", "content", "correctlabel", "description", "duration", "id", "interactiontype", "maxscore", "options", "optionslabel", "rawscore", "subcontent", "success", "timecreated", "track"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ModH5pactivityGetResultsResponseAttemptsInnerResultsInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in options (list)
        _items = []
        if self.options:
            for _item_options in self.options:
                if _item_options:
                    _items.append(_item_options.to_dict())
            _dict['options'] = _items
        # set to None if answerlabel (nullable) is None
        # and model_fields_set contains the field
        if self.answerlabel is None and "answerlabel" in self.model_fields_set:
            _dict['answerlabel'] = None

        # set to None if attemptid (nullable) is None
        # and model_fields_set contains the field
        if self.attemptid is None and "attemptid" in self.model_fields_set:
            _dict['attemptid'] = None

        # set to None if completion (nullable) is None
        # and model_fields_set contains the field
        if self.completion is None and "completion" in self.model_fields_set:
            _dict['completion'] = None

        # set to None if content (nullable) is None
        # and model_fields_set contains the field
        if self.content is None and "content" in self.model_fields_set:
            _dict['content'] = None

        # set to None if correctlabel (nullable) is None
        # and model_fields_set contains the field
        if self.correctlabel is None and "correctlabel" in self.model_fields_set:
            _dict['correctlabel'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if duration (nullable) is None
        # and model_fields_set contains the field
        if self.duration is None and "duration" in self.model_fields_set:
            _dict['duration'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if interactiontype (nullable) is None
        # and model_fields_set contains the field
        if self.interactiontype is None and "interactiontype" in self.model_fields_set:
            _dict['interactiontype'] = None

        # set to None if maxscore (nullable) is None
        # and model_fields_set contains the field
        if self.maxscore is None and "maxscore" in self.model_fields_set:
            _dict['maxscore'] = None

        # set to None if optionslabel (nullable) is None
        # and model_fields_set contains the field
        if self.optionslabel is None and "optionslabel" in self.model_fields_set:
            _dict['optionslabel'] = None

        # set to None if rawscore (nullable) is None
        # and model_fields_set contains the field
        if self.rawscore is None and "rawscore" in self.model_fields_set:
            _dict['rawscore'] = None

        # set to None if subcontent (nullable) is None
        # and model_fields_set contains the field
        if self.subcontent is None and "subcontent" in self.model_fields_set:
            _dict['subcontent'] = None

        # set to None if success (nullable) is None
        # and model_fields_set contains the field
        if self.success is None and "success" in self.model_fields_set:
            _dict['success'] = None

        # set to None if timecreated (nullable) is None
        # and model_fields_set contains the field
        if self.timecreated is None and "timecreated" in self.model_fields_set:
            _dict['timecreated'] = None

        # set to None if track (nullable) is None
        # and model_fields_set contains the field
        if self.track is None and "track" in self.model_fields_set:
            _dict['track'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ModH5pactivityGetResultsResponseAttemptsInnerResultsInner from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "ModH5pactivityGetResultsResponseAttemptsInnerResultsInner" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "answerlabel": obj.get("answerlabel"),
            "attemptid": obj.get("attemptid"),
            "completion": obj.get("completion"),
            "content": obj.get("content"),
            "correctlabel": obj.get("correctlabel"),
            "description": obj.get("description"),
            "duration": obj.get("duration") if obj.get("duration") is not None else 0,
            "id": obj.get("id"),
            "interactiontype": obj.get("interactiontype"),
            "maxscore": obj.get("maxscore"),
            "options": [ModH5pactivityGetResultsResponseAttemptsInnerResultsInnerOptionsInner.from_dict(_item) for _item in obj["options"]] if obj.get("options") is not None else None,
            "optionslabel": obj.get("optionslabel"),
            "rawscore": obj.get("rawscore"),
            "subcontent": obj.get("subcontent"),
            "success": obj.get("success"),
            "timecreated": obj.get("timecreated"),
            "track": obj.get("track")
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from poodle_async_full.models.mod_quiz_get_attempt_data_response_attempt_gradeitemmarks_inner import ModQuizGetAttemptDataResponseAttemptGradeitemmarksInner
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class ModQuizGetAttemptDataResponseAttempt(BaseModel):
    """
    ModQuizGetAttemptDataResponseAttempt
    """ # noqa: E501
    attempt: Optional[StrictInt] = Field(default=None, description="Sequentially numbers this students attempts at this quiz.")
    currentpage: Optional[StrictInt] = Field(default=None, description="Attempt current page.")
    gradednotificationsenttime: Optional[StrictInt] = Field(default=None, description="Time when the student was notified that manual grading of their attempt was complete.")
    gradeitemmarks: Optional[List[ModQuizGetAttemptDataResponseAttemptGradeitemmarksInner]] = Field(default=None, description="If the quiz has additional grades set up, the mark for each grade for this attempt.")
    id: Optional[StrictInt] = Field(default=None, description="Attempt id.")
    layout: Optional[StrictStr] = Field(default=None, description="Attempt layout.")
    preview: Optional[StrictInt] = Field(default=None, description="Whether is a preview attempt or not.")
    quiz: Optional[StrictInt] = Field(default=None, description="Foreign key reference to the quiz that was attempted.")
    state: Optional[StrictStr] = Field(default=None, description="The current state of the attempts. 'inprogress',                                                 'overdue', 'finished' or 'abandoned'.")
    sumgrades: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Total marks for this attempt.")
    timecheckstate: Optional[StrictInt] = Field(default=None, description="Next time quiz cron should check attempt for                                                         state changes.  NULL means never check.")
    timefinish: Optional[StrictInt] = Field(default=None, description="Time when the attempt was submitted.                                                     0 if the attempt has not been submitted yet.")
    timemodified: Optional[StrictInt] = Field(default=None, description="Last modified time.")
    timemodifiedoffline: Optional[StrictInt] = Field(default=None, description="Last modified time via webservices.")
    timestart: Optional[StrictInt] = Field(default=None, description="Time when the attempt was started.")
    uniqueid: Optional[StrictInt] = Field(default=None, description="Foreign key reference to the question_usage that holds the                                                     details of the the question_attempts that make up this quiz                                                     attempt.")
    userid: Optional[StrictInt] = Field(default=None, description="Foreign key reference to the user whose attempt this is.")
    __properties: ClassVar[List[str]] = ["attempt", "currentpage", "gradednotificationsenttime", "gradeitemmarks", "id", "layout", "preview", "quiz", "state", "sumgrades", "timecheckstate", "timefinish", "timemodified", "timemodifiedoffline", "timestart", "uniqueid", "userid"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ModQuizGetAttemptDataResponseAttempt from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in gradeitemmarks (list)
        _items = []
        if self.gradeitemmarks:
            for _item_gradeitemmarks in self.gradeitemmarks:
                if _item_gradeitemmarks:
                    _items.append(_item_gradeitemmarks.to_dict())
            _dict['gradeitemmarks'] = _items
        # set to None if attempt (nullable) is None
        # and model_fields_set contains the field
        if self.attempt is None and "attempt" in self.model_fields_set:
            _dict['attempt'] = None

        # set to None if currentpage (nullable) is None
        # and model_fields_set contains the field
        if self.currentpage is None and "currentpage" in self.model_fields_set:
            _dict['currentpage'] = None

        # set to None if gradednotificationsenttime (nullable) is None
        # and model_fields_set contains the field
        if self.gradednotificationsenttime is None and "gradednotificationsenttime" in self.model_fields_set:
            _dict['gradednotificationsenttime'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if layout (nullable) is None
        # and model_fields_set contains the field
        if self.layout is None and "layout" in self.model_fields_set:
            _dict['layout'] = None

        # set to None if preview (nullable) is None
        # and model_fields_set contains the field
        if self.preview is None and "preview" in self.model_fields_set:
            _dict['preview'] = None

        # set to None if quiz (nullable) is None
        # and model_fields_set contains the field
        if self.quiz is None and "quiz" in self.model_fields_set:
            _dict['quiz'] = None

        # set to None if state (nullable) is None
        # and model_fields_set contains the field
        if self.state is None and "state" in self.model_fields_set:
            _dict['state'] = None

        # set to None if sumgrades (nullable) is None
        # and model_fields_set contains the field
        if self.sumgrades is None and "sumgrades" in self.model_fields_set:
            _dict['sumgrades'] = None

        # set to None if timecheckstate (nullable) is None
        # and model_fields_set contains the field
        if self.timecheckstate is None and "timecheckstate" in self.model_fields_set:
            _dict['timecheckstate'] = None

        # set to None if timefinish (nullable) is None
        # and model_fields_set contains the field
        if self.timefinish is None and "timefinish" in self.model_fields_set:
            _dict['timefinish'] = None

        # set to None if timemodified (nullable) is None
        # and model_fields_set contains the field
        if self.timemodified is None and "timemodified" in self.model_fields_set:
            _dict['timemodified'] = None

        # set to None if timemodifiedoffline (nullable) is None
        # and model_fields_set contains the field
        if self.timemodifiedoffline is None and "timemodifiedoffline" in self.model_fields_set:
            _dict['timemodifiedoffline'] = None

        # set to None if timestart (nullable) is None
        # and model_fields_set contains the field
        if self.timestart is None and "timestart" in self.model_fields_set:
            _dict['timestart'] = None

        # set to None if uniqueid (nullable) is None
        # and model_fields_set contains the field
        if self.uniqueid is None and "uniqueid" in self.model_fields_set:
            _dict['uniqueid'] = None

        # set to None if userid (nullable) is None
        # and model_fields_set contains the field
        if self.userid is None and "userid" in self.model_fields_set:
            _dict['userid'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ModQuizGetAttemptDataResponseAttempt from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "ModQuizGetAttemptDataResponseAttempt" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "attempt": obj.get("attempt"),
            "currentpage": obj.get("currentpage"),
            "gradednotificationsenttime": obj.get("gradednotificationsenttime"),
            "gradeitemmarks": [ModQuizGetAttemptDataResponseAttemptGradeitemmarksInner.from_dict(_item) for _item in obj["gradeitemmarks"]] if obj.get("gradeitemmarks") is not None else None,
            "id": obj.get("id"),
            "layout": obj.get("layout"),
            "preview": obj.get("preview"),
            "quiz": obj.get("quiz"),
            "state": obj.get("state"),
            "sumgrades": obj.get("sumgrades"),
            "timecheckstate": obj.get("timecheckstate"),
            "timefinish": obj.get("timefinish"),
            "timemodified": obj.get("timemodified"),
            "timemodifiedoffline": obj.get("timemodifiedoffline"),
            "timestart": obj.get("timestart"),
            "uniqueid": obj.get("uniqueid"),
            "userid": obj.get("userid")
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class CoreCompetencyDuplicateCompetencyFrameworkResponse(BaseModel):
    """
    CoreCompetencyDuplicateCompetencyFrameworkResponse
    """ # noqa: E501
    canmanage: StrictBool = Field(description="canmanage")
    competenciescount: StrictInt = Field(description="competenciescount")
    contextid: StrictInt = Field(description="contextid")
    contextname: StrictStr = Field(description="contextname")
    contextnamenoprefix: StrictStr = Field(description="contextnamenoprefix")
    description: StrictStr = Field(description="description")
    descriptionformat: Optional[StrictInt] = Field(default=1, description="description format (1 = HTML, 0 = MOODLE, 2 = PLAIN, or 4 = MARKDOWN)")
    id: StrictInt = Field(description="id")
    idnumber: StrictStr = Field(description="idnumber")
    scaleconfiguration: StrictStr = Field(description="scaleconfiguration")
    scaleid: StrictInt = Field(description="scaleid")
    shortname: StrictStr = Field(description="shortname")
    taxonomies: StrictStr = Field(description="taxonomies")
    timecreated: StrictInt = Field(description="timecreated")
    timemodified: StrictInt = Field(description="timemodified")
    usermodified: StrictInt = Field(description="usermodified")
    visible: StrictBool = Field(description="visible")
    __properties: ClassVar[List[str]] = ["canmanage", "competenciescount", "contextid", "contextname", "contextnamenoprefix", "description", "descriptionformat", "id", "idnumber", "scaleconfiguration", "scaleid", "shortname", "taxonomies", "timecreated", "timemodified", "usermodified", "visible"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CoreCompetencyDuplicateCompetencyFrameworkResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if descriptionformat (nullable) is None
        # and model_fields_set contains the field
        if self.descriptionformat is None and "descriptionformat" in self.model_fields_set:
            _dict['descriptionformat'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CoreCompetencyDuplicateCompetencyFrameworkResponse from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "CoreCompetencyDuplicateCompetencyFrameworkResponse" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "canmanage": obj.get("canmanage") if obj.get("canmanage") is not None else False,
            "competenciescount": obj.get("competenciescount") if obj.get("competenciescount") is not None else 0,
            "contextid": obj.get("contextid") if obj.get("contextid") is not None else 0,
            "contextname": obj.get("contextname") if obj.get("contextname") is not None else '',
            "contextnamenoprefix": obj.get("contextnamenoprefix") if obj.get("contextnamenoprefix") is not None else '',
            "description": obj.get("description") if obj.get("description") is not None else '',
            "descriptionformat": obj.get("descriptionformat") if obj.get("descriptionformat") is not None else 1,
            "id": obj.get("id") if obj.get("id") is not None else 0,
            "idnumber": obj.get("idnumber") if obj.get("idnumber") is not None else '',
            "scaleconfiguration": obj.get("scaleconfiguration") if obj.get("scaleconfiguration") is not None else '',
            "scaleid": obj.get("scaleid") if obj.get("scaleid") is not None else 0,
            "shortname": obj.get("shortname") if obj.get("shortname") is not None else '',
            "taxonomies": obj.get("taxonomies") if obj.get("taxonomies") is not None else '',
            "timecreated": obj.get("timecreated") if obj.get("timecreated") is not None else 0,
            "timemodified": obj.get("timemodified") if obj.get("timemodified") is not None else 0,
            "usermodified": obj.get("usermodified") if obj.get("usermodified") is not None else 0,
            "visible": obj.get("visible") if obj.get("visible") is not None else True
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List
from poodle_async_full.models.block_starredcourses_get_starred_courses_response_inner import BlockStarredcoursesGetStarredCoursesResponseInner
from poodle_async_full.models.core_competency_list_competencies_in_template_response_inner import CoreCompetencyListCompetenciesInTemplateResponseInner
from poodle_async_full.models.core_competency_list_competency_frameworks_response_inner import CoreCompetencyListCompetencyFrameworksResponseInner
from poodle_async_full.models.report_competency_data_for_report_response_usercompetencies_inner_competency_comppath import ReportCompetencyDataForReportResponseUsercompetenciesInnerCompetencyComppath
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class ToolLpDataForCompetencySummaryResponse(BaseModel):
    """
    ToolLpDataForCompetencySummaryResponse
    """ # noqa: E501
    competency: CoreCompetencyListCompetenciesInTemplateResponseInner
    comppath: ReportCompetencyDataForReportResponseUsercompetenciesInnerCompetencyComppath
    framework: CoreCompetencyListCompetencyFrameworksResponseInner
    hascourses: StrictBool = Field(description="hascourses")
    hasrelatedcompetencies: StrictBool = Field(description="hasrelatedcompetencies")
    linkedcourses: List[BlockStarredcoursesGetStarredCoursesResponseInner] = Field(description="linkedcourses")
    pluginbaseurl: StrictStr = Field(description="pluginbaseurl")
    relatedcompetencies: List[CoreCompetencyListCompetenciesInTemplateResponseInner] = Field(description="relatedcompetencies")
    scaleconfiguration: StrictStr = Field(description="scaleconfiguration")
    scaleid: StrictInt = Field(description="scaleid")
    taxonomyterm: StrictStr = Field(description="taxonomyterm")
    __properties: ClassVar[List[str]] = ["competency", "comppath", "framework", "hascourses", "hasrelatedcompetencies", "linkedcourses", "pluginbaseurl", "relatedcompetencies", "scaleconfiguration", "scaleid", "taxonomyterm"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ToolLpDataForCompetencySummaryResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of competency
        if self.competency:
            _dict['competency'] = self.competency.to_dict()
        # override the default output from pydantic by calling `to_dict()` of comppath
        if self.comppath:
            _dict['comppath'] = self.comppath.to_dict()
        # override the default output from pydantic by calling `to_dict()` of framework
        if self.framework:
            _dict['framework'] = self.framework.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in linkedcourses (list)
        _items = []
        if self.linkedcourses:
            for _item_linkedcourses in self.linkedcourses:
                if _item_linkedcourses:
                    _items.append(_item_linkedcourses.to_dict())
            _dict['linkedcourses'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in relatedcompetencies (list)
        _items = []
        if self.relatedcompetencies:
            for _item_relatedcompetencies in self.relatedcompetencies:
                if _item_relatedcompetencies:
                    _items.append(_item_relatedcompetencies.to_dict())
            _dict['relatedcompetencies'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ToolLpDataForCompetencySummaryResponse from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "ToolLpDataForCompetencySummaryResponse" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "competency": CoreCompetencyListCompetenciesInTemplateResponseInner.from_dict(obj["competency"]) if obj.get("competency") is not None else None,
            "comppath": ReportCompetencyDataForReportResponseUsercompetenciesInnerCompetencyComppath.from_dict(obj["comppath"]) if obj.get("comppath") is not None else None,
            "framework": CoreCompetencyListCompetencyFrameworksResponseInner.from_dict(obj["framework"]) if obj.get("framework") is not None else None,
            "hascourses": obj.get("hascourses") if obj.get("hascourses") is not None else False,
            "hasrelatedcompetencies": obj.get("hasrelatedcompetencies") if obj.get("hasrelatedcompetencies") is not None else False,
            "linkedcourses": [BlockStarredcoursesGetStarredCoursesResponseInner.from_dict(_item) for _item in obj["linkedcourses"]] if obj.get("linkedcourses") is not None else None,
            "pluginbaseurl": obj.get("pluginbaseurl") if obj.get("pluginbaseurl") is not None else '',
            "relatedcompetencies": [CoreCompetencyListCompetenciesInTemplateResponseInner.from_dict(_item) for _item in obj["relatedcompetencies"]] if obj.get("relatedcompetencies") is not None else None,
            "scaleconfiguration": obj.get("scaleconfiguration") if obj.get("scaleconfiguration") is not None else '',
            "scaleid": obj.get("scaleid") if obj.get("scaleid") is not None else 0,
            "taxonomyterm": obj.get("taxonomyterm") if obj.get("taxonomyterm") is not None else ''
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from poodle_async_full.models.core_badges_get_badge_response_badge_alignment_inner import CoreBadgesGetBadgeResponseBadgeAlignmentInner
from poodle_async_full.models.core_badges_get_user_badge_by_hash_response_badge_inner_endorsement import CoreBadgesGetUserBadgeByHashResponseBadgeInnerEndorsement
from poodle_async_full.models.core_badges_get_user_badge_by_hash_response_badge_inner_relatedbadges_inner import CoreBadgesGetUserBadgeByHashResponseBadgeInnerRelatedbadgesInner
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class CoreBadgesGetUserBadgeByHashResponseBadgeInner(BaseModel):
    """
    CoreBadgesGetUserBadgeByHashResponseBadgeInner
    """ # noqa: E501
    alignment: List[CoreBadgesGetBadgeResponseBadgeAlignmentInner] = Field(description="Badge alignments")
    attachment: Optional[StrictInt] = Field(default=0, description="Attachment")
    badgeurl: StrictStr = Field(description="Badge URL")
    coursefullname: Optional[StrictStr] = Field(default='', description="Full name of the course")
    courseid: Optional[StrictInt] = Field(default=None, description="Course id")
    dateexpire: Optional[StrictInt] = Field(description="Date expire")
    dateissued: StrictInt = Field(description="Date issued")
    description: Optional[StrictStr] = Field(description="Badge description")
    email: Optional[StrictStr] = Field(default='', description="User email")
    endorsement: Optional[CoreBadgesGetUserBadgeByHashResponseBadgeInnerEndorsement] = None
    expiredate: Optional[StrictInt] = Field(default=None, description="Expire date")
    expireperiod: Optional[StrictInt] = Field(default=None, description="Expire period")
    id: Optional[StrictInt] = Field(default=0, description="Badge id")
    imagecaption: Optional[StrictStr] = Field(default=None, description="Caption of the image")
    issuedid: Optional[StrictInt] = Field(default=0, description="Issued id")
    issuercontact: Optional[StrictStr] = Field(description="Issuer contact")
    issuername: StrictStr = Field(description="Issuer name")
    issuerurl: StrictStr = Field(description="Issuer URL")
    language: Optional[StrictStr] = Field(default=None, description="Language")
    message: Optional[StrictStr] = Field(default='', description="Message")
    messagesubject: Optional[StrictStr] = Field(default='', description="Message subject")
    name: StrictStr = Field(description="Badge name")
    nextcron: Optional[StrictInt] = Field(default=None, description="Next cron")
    notification: Optional[StrictInt] = Field(default=0, description="Whether to notify when badge is awarded")
    recipientfullname: StrictStr = Field(description="Full name of the awarded user")
    recipientid: StrictInt = Field(description="Id of the awarded user")
    relatedbadges: List[CoreBadgesGetUserBadgeByHashResponseBadgeInnerRelatedbadgesInner] = Field(description="Related badges")
    status: Optional[StrictInt] = Field(default=0, description="Status")
    timecreated: Optional[StrictInt] = Field(default=0, description="Time created")
    timemodified: Optional[StrictInt] = Field(default=0, description="Time modified")
    type: Optional[StrictInt] = Field(default=0, description="Type")
    uniquehash: StrictStr = Field(description="Unique hash")
    usercreated: Optional[StrictInt] = Field(default=0, description="User created")
    usermodified: Optional[StrictInt] = Field(default=0, description="User modified")
    version: Optional[StrictStr] = Field(default=None, description="Version")
    visible: Optional[StrictInt] = Field(default=0, description="Visible")
    __properties: ClassVar[List[str]] = ["alignment", "attachment", "badgeurl", "coursefullname", "courseid", "dateexpire", "dateissued", "description", "email", "endorsement", "expiredate", "expireperiod", "id", "imagecaption", "issuedid", "issuercontact", "issuername", "issuerurl", "language", "message", "messagesubject", "name", "nextcron", "notification", "recipientfullname", "recipientid", "relatedbadges", "status", "timecreated", "timemodified", "type", "uniquehash", "usercreated", "usermodified", "version", "visible"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CoreBadgesGetUserBadgeByHashResponseBadgeInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in alignment (list)
        _items = []
        if self.alignment:
            for _item_alignment in self.alignment:
                if _item_alignment:
                    _items.append(_item_alignment.to_dict())
            _dict['alignment'] = _items
        # override the default output from pydantic by calling `to_dict()` of endorsement
        if self.endorsement:
            _dict['endorsement'] = self.endorsement.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in relatedbadges (list)
        _items = []
        if self.relatedbadges:
            for _item_relatedbadges in self.relatedbadges:
                if _item_relatedbadges:
                    _items.append(_item_relatedbadges.to_dict())
            _dict['relatedbadges'] = _items
        # set to None if courseid (nullable) is None
        # and model_fields_set contains the field
        if self.courseid is None and "courseid" in self.model_fields_set:
            _dict['courseid'] = None

        # set to None if dateexpire (nullable) is None
        # and model_fields_set contains the field
        if self.dateexpire is None and "dateexpire" in self.model_fields_set:
            _dict['dateexpire'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if expiredate (nullable) is None
        # and model_fields_set contains the field
        if self.expiredate is None and "expiredate" in self.model_fields_set:
            _dict['expiredate'] = None

        # set to None if expireperiod (nullable) is None
        # and model_fields_set contains the field
        if self.expireperiod is None and "expireperiod" in self.model_fields_set:
            _dict['expireperiod'] = None

        # set to None if imagecaption (nullable) is None
        # and model_fields_set contains the field
        if self.imagecaption is None and "imagecaption" in self.model_fields_set:
            _dict['imagecaption'] = None

        # set to None if issuercontact (nullable) is None
        # and model_fields_set contains the field
        if self.issuercontact is None and "issuercontact" in self.model_fields_set:
            _dict['issuercontact'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['language'] = None

        # set to None if nextcron (nullable) is None
        # and model_fields_set contains the field
        if self.nextcron is None and "nextcron" in self.model_fields_set:
            _dict['nextcron'] = None

        # set to None if version (nullable) is None
        # and model_fields_set contains the field
        if self.version is None and "version" in self.model_fields_set:
            _dict['version'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CoreBadgesGetUserBadgeByHashResponseBadgeInner from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "CoreBadgesGetUserBadgeByHashResponseBadgeInner" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "alignment": [CoreBadgesGetBadgeResponseBadgeAlignmentInner.from_dict(_item) for _item in obj["alignment"]] if obj.get("alignment") is not None else None,
            "attachment": obj.get("attachment") if obj.get("attachment") is not None else 0,
            "badgeurl": obj.get("badgeurl") if obj.get("badgeurl") is not None else '',
            "coursefullname": obj.get("coursefullname") if obj.get("coursefullname") is not None else '',
            "courseid": obj.get("courseid"),
            "dateexpire": obj.get("dateexpire"),
            "dateissued": obj.get("dateissued") if obj.get("dateissued") is not None else 0,
            "description": obj.get("description"),
            "email": obj.get("email") if obj.get("email") is not None else '',
            "endorsement": CoreBadgesGetUserBadgeByHashResponseBadgeInnerEndorsement.from_dict(obj["endorsement"]) if obj.get("endorsement") is not None else None,
            "expiredate": obj.get("expiredate"),
            "expireperiod": obj.get("expireperiod"),
            "id": obj.get("id") if obj.get("id") is not None else 0,
            "imagecaption": obj.get("imagecaption"),
            "issuedid": obj.get("issuedid") if obj.get("issuedid") is not None else 0,
            "issuercontact": obj.get("issuercontact"),
            "issuername": obj.get("issuername") if obj.get("issuername") is not None else '',
            "issuerurl": obj.get("issuerurl") if obj.get("issuerurl") is not None else '',
            "language": obj.get("language"),
            "message": obj.get("message") if obj.get("message") is not None else '',
            "messagesubject": obj.get("messagesubject") if obj.get("messagesubject") is not None else '',
            "name": obj.get("name") if obj.get("name") is not None else '',
            "nextcron": obj.get("nextcron"),
            "notification": obj.get("notification") if obj.get("notification") is not None else 0,
            "recipientfullname": obj.get("recipientfullname") if obj.get("recipientfullname") is not None else '',
            "recipientid": obj.get("recipientid") if obj.get("recipientid") is not None else 0,
            "relatedbadges": [CoreBadgesGetUserBadgeByHashResponseBadgeInnerRelatedbadgesInner.from_dict(_item) for _item in obj["relatedbadges"]] if obj.get("relatedbadges") is not None else None,
            "status": obj.get("status") if obj.get("status") is not None else 0,
            "timecreated": obj.get("timecreated") if obj.get("timecreated") is not None else 0,
            "timemodified": obj.get("timemodified") if obj.get("timemodified") is not None else 0,
            "type": obj.get("type") if obj.get("type") is not None else 0,
            "uniquehash": obj.get("uniquehash") if obj.get("uniquehash") is not None else '',
            "usercreated": obj.get("usercreated") if obj.get("usercreated") is not None else 0,
            "usermodified": obj.get("usermodified") if obj.get("usermodified") is not None else 0,
            "version": obj.get("version"),
            "visible": obj.get("visible") if obj.get("visible") is not None else 0
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from poodle_async_full.models.core_block_get_course_blocks_response_blocks_inner_contents_files_inner import CoreBlockGetCourseBlocksResponseBlocksInnerContentsFilesInner
from poodle_async_full.models.core_cohort_get_cohorts_response_inner_customfields_inner import CoreCohortGetCohortsResponseInnerCustomfieldsInner
from poodle_async_full.models.core_course_get_courses_by_field_response_courses_inner_contacts_inner import CoreCourseGetCoursesByFieldResponseCoursesInnerContactsInner
from poodle_async_full.models.core_course_get_courses_by_field_response_courses_inner_courseformatoptions_inner import CoreCourseGetCoursesByFieldResponseCoursesInnerCourseformatoptionsInner
from poodle_async_full.models.core_course_get_courses_by_field_response_courses_inner_filters_inner import CoreCourseGetCoursesByFieldResponseCoursesInnerFiltersInner
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class CoreCourseGetCoursesByFieldResponseCoursesInner(BaseModel):
    """
    CoreCourseGetCoursesByFieldResponseCoursesInner
    """ # noqa: E501
    cacherev: Optional[StrictInt] = Field(default=None, description="Cache revision number")
    calendartype: Optional[StrictStr] = Field(default=None, description="Calendar type")
    categoryid: Optional[StrictInt] = Field(description="category id")
    categoryname: Optional[StrictStr] = Field(description="category name")
    communicationroomname: Optional[StrictStr] = Field(default=None, description="Communication tool room name.")
    communicationroomurl: Optional[StrictStr] = Field(default=None, description="Communication tool room URL.")
    completionnotify: Optional[StrictInt] = Field(default=None, description="1: yes 0: no")
    contacts: List[CoreCourseGetCoursesByFieldResponseCoursesInnerContactsInner] = Field(description="contact users")
    courseformatoptions: Optional[List[CoreCourseGetCoursesByFieldResponseCoursesInnerCourseformatoptionsInner]] = Field(default=None, description="Additional options for particular course format.")
    courseimage: Optional[StrictStr] = Field(default=None, description="Course image")
    customfields: Optional[List[CoreCohortGetCohortsResponseInnerCustomfieldsInner]] = Field(default=None, description="Custom fields")
    defaultgroupingid: Optional[StrictInt] = Field(default=None, description="default grouping id")
    displayname: Optional[StrictStr] = Field(description="course display name")
    enablecompletion: Optional[StrictInt] = Field(default=None, description="Completion enabled? 1: yes 0: no")
    enddate: Optional[StrictInt] = Field(default=None, description="Timestamp when the course end")
    enrollmentmethods: List[Optional[StrictStr]] = Field(description="enrollment methods list")
    filters: Optional[List[CoreCourseGetCoursesByFieldResponseCoursesInnerFiltersInner]] = Field(default=None, description="Course filters")
    format: Optional[StrictStr] = Field(default=None, description="Course format: weeks, topics, social, site,..")
    fullname: Optional[StrictStr] = Field(description="course full name")
    groupmode: Optional[StrictInt] = Field(default=None, description="no group, separate, visible")
    groupmodeforce: Optional[StrictInt] = Field(default=None, description="1: yes, 0: no")
    id: Optional[StrictInt] = Field(description="course id")
    idnumber: Optional[StrictStr] = Field(default=None, description="Id number")
    lang: Optional[StrictStr] = Field(default=None, description="Forced course language")
    legacyfiles: Optional[StrictInt] = Field(default=None, description="If legacy files are enabled")
    marker: Optional[StrictInt] = Field(default=None, description="Current course marker")
    maxbytes: Optional[StrictInt] = Field(default=None, description="Largest size of file that can be uploaded into")
    newsitems: Optional[StrictInt] = Field(default=None, description="Number of recent items appearing on the course page")
    overviewfiles: List[CoreBlockGetCourseBlocksResponseBlocksInnerContentsFilesInner] = Field(description="additional overview files attached to this course")
    requested: Optional[StrictInt] = Field(default=None, description="If is a requested course")
    shortname: Optional[StrictStr] = Field(description="course short name")
    showactivitydates: Optional[StrictBool] = Field(description="Whether the activity dates are shown or not")
    showcompletionconditions: Optional[StrictBool] = Field(description="Whether the activity completion conditions are shown or not")
    showgrades: Optional[StrictInt] = Field(default=None, description="1 if grades are shown, otherwise 0")
    showreports: Optional[StrictInt] = Field(default=None, description="Are activity report shown (yes = 1, no =0)")
    sortorder: Optional[StrictInt] = Field(default=None, description="Sort order in the category")
    startdate: Optional[StrictInt] = Field(default=None, description="Timestamp when the course start")
    summary: Optional[StrictStr] = Field(description="summary")
    summaryfiles: Optional[List[CoreBlockGetCourseBlocksResponseBlocksInnerContentsFilesInner]] = Field(default=None, description="summary files in the summary field")
    summaryformat: Optional[StrictInt] = Field(description="summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN, or 4 = MARKDOWN)")
    theme: Optional[StrictStr] = Field(default=None, description="Fame of the forced theme")
    timecreated: Optional[StrictInt] = Field(default=None, description="Time when the course was created")
    timemodified: Optional[StrictInt] = Field(default=None, description="Last time  the course was updated")
    visible: Optional[StrictInt] = Field(default=None, description="1: available to student, 0:not available")
    __properties: ClassVar[List[str]] = ["cacherev", "calendartype", "categoryid", "categoryname", "communicationroomname", "communicationroomurl", "completionnotify", "contacts", "courseformatoptions", "courseimage", "customfields", "defaultgroupingid", "displayname", "enablecompletion", "enddate", "enrollmentmethods", "filters", "format", "fullname", "groupmode", "groupmodeforce", "id", "idnumber", "lang", "legacyfiles", "marker", "maxbytes", "newsitems", "overviewfiles", "requested", "shortname", "showactivitydates", "showcompletionconditions", "showgrades", "showreports", "sortorder", "startdate", "summary", "summaryfiles", "summaryformat", "theme", "timecreated", "timemodified", "visible"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CoreCourseGetCoursesByFieldResponseCoursesInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in contacts (list)
        _items = []
        if self.contacts:
            for _item_contacts in self.contacts:
                if _item_contacts:
                    _items.append(_item_contacts.to_dict())
            _dict['contacts'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in courseformatoptions (list)
        _items = []
        if self.courseformatoptions:
            for _item_courseformatoptions in self.courseformatoptions:
                if _item_courseformatoptions:
                    _items.append(_item_courseformatoptions.to_dict())
            _dict['courseformatoptions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in customfields (list)
        _items = []
        if self.customfields:
            for _item_customfields in self.customfields:
                if _item_customfields:
                    _items.append(_item_customfields.to_dict())
            _dict['customfields'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in filters (list)
        _items = []
        if self.filters:
            for _item_filters in self.filters:
                if _item_filters:
                    _items.append(_item_filters.to_dict())
            _dict['filters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in overviewfiles (list)
        _items = []
        if self.overviewfiles:
            for _item_overviewfiles in self.overviewfiles:
                if _item_overviewfiles:
                    _items.append(_item_overviewfiles.to_dict())
            _dict['overviewfiles'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in summaryfiles (list)
        _items = []
        if self.summaryfiles:
            for _item_summaryfiles in self.summaryfiles:
                if _item_summaryfiles:
                    _items.append(_item_summaryfiles.to_dict())
            _dict['summaryfiles'] = _items
        # set to None if cacherev (nullable) is None
        # and model_fields_set contains the field
        if self.cacherev is None and "cacherev" in self.model_fields_set:
            _dict['cacherev'] = None

        # set to None if calendartype (nullable) is None
        # and model_fields_set contains the field
        if self.calendartype is None and "calendartype" in self.model_fields_set:
            _dict['calendartype'] = None

        # set to None if categoryid (nullable) is None
        # and model_fields_set contains the field
        if self.categoryid is None and "categoryid" in self.model_fields_set:
            _dict['categoryid'] = None

        # set to None if categoryname (nullable) is None
        # and model_fields_set contains the field
        if self.categoryname is None and "categoryname" in self.model_fields_set:
            _dict['categoryname'] = None

        # set to None if communicationroomname (nullable) is None
        # and model_fields_set contains the field
        if self.communicationroomname is None and "communicationroomname" in self.model_fields_set:
            _dict['communicationroomname'] = None

        # set to None if communicationroomurl (nullable) is None
        # and model_fields_set contains the field
        if self.communicationroomurl is None and "communicationroomurl" in self.model_fields_set:
            _dict['communicationroomurl'] = None

        # set to None if completionnotify (nullable) is None
        # and model_fields_set contains the field
        if self.completionnotify is None and "completionnotify" in self.model_fields_set:
            _dict['completionnotify'] = None

        # set to None if courseimage (nullable) is None
        # and model_fields_set contains the field
        if self.courseimage is None and "courseimage" in self.model_fields_set:
            _dict['courseimage'] = None

        # set to None if defaultgroupingid (nullable) is None
        # and model_fields_set contains the field
        if self.defaultgroupingid is None and "defaultgroupingid" in self.model_fields_set:
            _dict['defaultgroupingid'] = None

        # set to None if displayname (nullable) is None
        # and model_fields_set contains the field
        if self.displayname is None and "displayname" in self.model_fields_set:
            _dict['displayname'] = None

        # set to None if enablecompletion (nullable) is None
        # and model_fields_set contains the field
        if self.enablecompletion is None and "enablecompletion" in self.model_fields_set:
            _dict['enablecompletion'] = None

        # set to None if enddate (nullable) is None
        # and model_fields_set contains the field
        if self.enddate is None and "enddate" in self.model_fields_set:
            _dict['enddate'] = None

        # set to None if format (nullable) is None
        # and model_fields_set contains the field
        if self.format is None and "format" in self.model_fields_set:
            _dict['format'] = None

        # set to None if fullname (nullable) is None
        # and model_fields_set contains the field
        if self.fullname is None and "fullname" in self.model_fields_set:
            _dict['fullname'] = None

        # set to None if groupmode (nullable) is None
        # and model_fields_set contains the field
        if self.groupmode is None and "groupmode" in self.model_fields_set:
            _dict['groupmode'] = None

        # set to None if groupmodeforce (nullable) is None
        # and model_fields_set contains the field
        if self.groupmodeforce is None and "groupmodeforce" in self.model_fields_set:
            _dict['groupmodeforce'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if idnumber (nullable) is None
        # and model_fields_set contains the field
        if self.idnumber is None and "idnumber" in self.model_fields_set:
            _dict['idnumber'] = None

        # set to None if lang (nullable) is None
        # and model_fields_set contains the field
        if self.lang is None and "lang" in self.model_fields_set:
            _dict['lang'] = None

        # set to None if legacyfiles (nullable) is None
        # and model_fields_set contains the field
        if self.legacyfiles is None and "legacyfiles" in self.model_fields_set:
            _dict['legacyfiles'] = None

        # set to None if marker (nullable) is None
        # and model_fields_set contains the field
        if self.marker is None and "marker" in self.model_fields_set:
            _dict['marker'] = None

        # set to None if maxbytes (nullable) is None
        # and model_fields_set contains the field
        if self.maxbytes is None and "maxbytes" in self.model_fields_set:
            _dict['maxbytes'] = None

        # set to None if newsitems (nullable) is None
        # and model_fields_set contains the field
        if self.newsitems is None and "newsitems" in self.model_fields_set:
            _dict['newsitems'] = None

        # set to None if requested (nullable) is None
        # and model_fields_set contains the field
        if self.requested is None and "requested" in self.model_fields_set:
            _dict['requested'] = None

        # set to None if shortname (nullable) is None
        # and model_fields_set contains the field
        if self.shortname is None and "shortname" in self.model_fields_set:
            _dict['shortname'] = None

        # set to None if showactivitydates (nullable) is None
        # and model_fields_set contains the field
        if self.showactivitydates is None and "showactivitydates" in self.model_fields_set:
            _dict['showactivitydates'] = None

        # set to None if showcompletionconditions (nullable) is None
        # and model_fields_set contains the field
        if self.showcompletionconditions is None and "showcompletionconditions" in self.model_fields_set:
            _dict['showcompletionconditions'] = None

        # set to None if showgrades (nullable) is None
        # and model_fields_set contains the field
        if self.showgrades is None and "showgrades" in self.model_fields_set:
            _dict['showgrades'] = None

        # set to None if showreports (nullable) is None
        # and model_fields_set contains the field
        if self.showreports is None and "showreports" in self.model_fields_set:
            _dict['showreports'] = None

        # set to None if sortorder (nullable) is None
        # and model_fields_set contains the field
        if self.sortorder is None and "sortorder" in self.model_fields_set:
            _dict['sortorder'] = None

        # set to None if startdate (nullable) is None
        # and model_fields_set contains the field
        if self.startdate is None and "startdate" in self.model_fields_set:
            _dict['startdate'] = None

        # set to None if summary (nullable) is None
        # and model_fields_set contains the field
        if self.summary is None and "summary" in self.model_fields_set:
            _dict['summary'] = None

        # set to None if summaryformat (nullable) is None
        # and model_fields_set contains the field
        if self.summaryformat is None and "summaryformat" in self.model_fields_set:
            _dict['summaryformat'] = None

        # set to None if theme (nullable) is None
        # and model_fields_set contains the field
        if self.theme is None and "theme" in self.model_fields_set:
            _dict['theme'] = None

        # set to None if timecreated (nullable) is None
        # and model_fields_set contains the field
        if self.timecreated is None and "timecreated" in self.model_fields_set:
            _dict['timecreated'] = None

        # set to None if timemodified (nullable) is None
        # and model_fields_set contains the field
        if self.timemodified is None and "timemodified" in self.model_fields_set:
            _dict['timemodified'] = None

        # set to None if visible (nullable) is None
        # and model_fields_set contains the field
        if self.visible is None and "visible" in self.model_fields_set:
            _dict['visible'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CoreCourseGetCoursesByFieldResponseCoursesInner from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "CoreCourseGetCoursesByFieldResponseCoursesInner" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "cacherev": obj.get("cacherev"),
            "calendartype": obj.get("calendartype"),
            "categoryid": obj.get("categoryid"),
            "categoryname": obj.get("categoryname"),
            "communicationroomname": obj.get("communicationroomname"),
            "communicationroomurl": obj.get("communicationroomurl"),
            "completionnotify": obj.get("completionnotify"),
            "contacts": [CoreCourseGetCoursesByFieldResponseCoursesInnerContactsInner.from_dict(_item) for _item in obj["contacts"]] if obj.get("contacts") is not None else None,
            "courseformatoptions": [CoreCourseGetCoursesByFieldResponseCoursesInnerCourseformatoptionsInner.from_dict(_item) for _item in obj["courseformatoptions"]] if obj.get("courseformatoptions") is not None else None,
            "courseimage": obj.get("courseimage"),
            "customfields": [CoreCohortGetCohortsResponseInnerCustomfieldsInner.from_dict(_item) for _item in obj["customfields"]] if obj.get("customfields") is not None else None,
            "defaultgroupingid": obj.get("defaultgroupingid"),
            "displayname": obj.get("displayname"),
            "enablecompletion": obj.get("enablecompletion"),
            "enddate": obj.get("enddate"),
            "enrollmentmethods": obj.get("enrollmentmethods"),
            "filters": [CoreCourseGetCoursesByFieldResponseCoursesInnerFiltersInner.from_dict(_item) for _item in obj["filters"]] if obj.get("filters") is not None else None,
            "format": obj.get("format"),
            "fullname": obj.get("fullname"),
            "groupmode": obj.get("groupmode"),
            "groupmodeforce": obj.get("groupmodeforce"),
            "id": obj.get("id"),
            "idnumber": obj.get("idnumber"),
            "lang": obj.get("lang"),
            "legacyfiles": obj.get("legacyfiles"),
            "marker": obj.get("marker"),
            "maxbytes": obj.get("maxbytes"),
            "newsitems": obj.get("newsitems"),
            "overviewfiles": [CoreBlockGetCourseBlocksResponseBlocksInnerContentsFilesInner.from_dict(_item) for _item in obj["overviewfiles"]] if obj.get("overviewfiles") is not None else None,
            "requested": obj.get("requested"),
            "shortname": obj.get("shortname"),
            "showactivitydates": obj.get("showactivitydates"),
            "showcompletionconditions": obj.get("showcompletionconditions"),
            "showgrades": obj.get("showgrades"),
            "showreports": obj.get("showreports"),
            "sortorder": obj.get("sortorder"),
            "startdate": obj.get("startdate"),
            "summary": obj.get("summary"),
            "summaryfiles": [CoreBlockGetCourseBlocksResponseBlocksInnerContentsFilesInner.from_dict(_item) for _item in obj["summaryfiles"]] if obj.get("summaryfiles") is not None else None,
            "summaryformat": obj.get("summaryformat"),
            "theme": obj.get("theme"),
            "timecreated": obj.get("timecreated"),
            "timemodified": obj.get("timemodified"),
            "visible": obj.get("visible")
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class ModChoiceGetChoiceOptionsResponseOptionsInner(BaseModel):
    """
    ModChoiceGetChoiceOptionsResponseOptionsInner
    """ # noqa: E501
    checked: Optional[StrictBool] = Field(description="we already answered")
    countanswers: Optional[StrictInt] = Field(description="number of answers")
    disabled: Optional[StrictBool] = Field(description="option disabled")
    displaylayout: Optional[StrictBool] = Field(description="true for orizontal, otherwise vertical")
    id: Optional[StrictInt] = Field(description="option id")
    maxanswers: Optional[StrictInt] = Field(description="maximum number of answers")
    text: Optional[StrictStr] = Field(description="text of the choice")
    __properties: ClassVar[List[str]] = ["checked", "countanswers", "disabled", "displaylayout", "id", "maxanswers", "text"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ModChoiceGetChoiceOptionsResponseOptionsInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if checked (nullable) is None
        # and model_fields_set contains the field
        if self.checked is None and "checked" in self.model_fields_set:
            _dict['checked'] = None

        # set to None if countanswers (nullable) is None
        # and model_fields_set contains the field
        if self.countanswers is None and "countanswers" in self.model_fields_set:
            _dict['countanswers'] = None

        # set to None if disabled (nullable) is None
        # and model_fields_set contains the field
        if self.disabled is None and "disabled" in self.model_fields_set:
            _dict['disabled'] = None

        # set to None if displaylayout (nullable) is None
        # and model_fields_set contains the field
        if self.displaylayout is None and "displaylayout" in self.model_fields_set:
            _dict['displaylayout'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['id'] = None

        # set to None if maxanswers (nullable) is None
        # and model_fields_set contains the field
        if self.maxanswers is None and "maxanswers" in self.model_fields_set:
            _dict['maxanswers'] = None

        # set to None if text (nullable) is None
        # and model_fields_set contains the field
        if self.text is None and "text" in self.model_fields_set:
            _dict['text'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ModChoiceGetChoiceOptionsResponseOptionsInner from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "ModChoiceGetChoiceOptionsResponseOptionsInner" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "checked": obj.get("checked"),
            "countanswers": obj.get("countanswers"),
            "disabled": obj.get("disabled"),
            "displaylayout": obj.get("displaylayout"),
            "id": obj.get("id"),
            "maxanswers": obj.get("maxanswers"),
            "text": obj.get("text")
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj





# coding: utf-8

"""
    Moodle Webservice API

    Auto-generated OpenAPI spec for Moodle's Webservice API.

    The version of the OpenAPI document: 5.0.3 (Build: 20251006)
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from poodle_async_full.models.core_competency_list_user_plans_response_inner import CoreCompetencyListUserPlansResponseInner
from poodle_async_full.models.tool_lp_data_for_plan_page_response_competencies_inner import ToolLpDataForPlanPageResponseCompetenciesInner
from typing import Optional, Set
from typing_extensions import Self

from poodle_async_full.configuration import settings

class ToolLpDataForPlanPageResponse(BaseModel):
    """
    ToolLpDataForPlanPageResponse
    """ # noqa: E501
    competencies: List[ToolLpDataForPlanPageResponseCompetenciesInner]
    competencycount: Optional[StrictInt] = Field(description="Count of competencies")
    contextid: Optional[StrictInt] = Field(description="Context ID.")
    plan: CoreCompetencyListUserPlansResponseInner
    pluginbaseurl: Optional[StrictStr] = Field(description="Plugin base URL.")
    proficientcompetencycount: Optional[StrictInt] = Field(description="Count of proficientcompetencies")
    proficientcompetencypercentage: Optional[Union[StrictFloat, StrictInt]] = Field(description="Percentage of competencies proficient")
    proficientcompetencypercentageformatted: Optional[StrictStr] = Field(description="Displayable percentage")
    __properties: ClassVar[List[str]] = ["competencies", "competencycount", "contextid", "plan", "pluginbaseurl", "proficientcompetencycount", "proficientcompetencypercentage", "proficientcompetencypercentageformatted"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ToolLpDataForPlanPageResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in competencies (list)
        _items = []
        if self.competencies:
            for _item_competencies in self.competencies:
                if _item_competencies:
                    _items.append(_item_competencies.to_dict())
            _dict['competencies'] = _items
        # override the default output from pydantic by calling `to_dict()` of plan
        if self.plan:
            _dict['plan'] = self.plan.to_dict()
        # set to None if competencycount (nullable) is None
        # and model_fields_set contains the field
        if self.competencycount is None and "competencycount" in self.model_fields_set:
            _dict['competencycount'] = None

        # set to None if contextid (nullable) is None
        # and model_fields_set contains the field
        if self.contextid is None and "contextid" in self.model_fields_set:
            _dict['contextid'] = None

        # set to None if pluginbaseurl (nullable) is None
        # and model_fields_set contains the field
        if self.pluginbaseurl is None and "pluginbaseurl" in self.model_fields_set:
            _dict['pluginbaseurl'] = None

        # set to None if proficientcompetencycount (nullable) is None
        # and model_fields_set contains the field
        if self.proficientcompetencycount is None and "proficientcompetencycount" in self.model_fields_set:
            _dict['proficientcompetencycount'] = None

        # set to None if proficientcompetencypercentage (nullable) is None
        # and model_fields_set contains the field
        if self.proficientcompetencypercentage is None and "proficientcompetencypercentage" in self.model_fields_set:
            _dict['proficientcompetencypercentage'] = None

        # set to None if proficientcompetencypercentageformatted (nullable) is None
        # and model_fields_set contains the field
        if self.proficientcompetencypercentageformatted is None and "proficientcompetencypercentageformatted" in self.model_fields_set:
            _dict['proficientcompetencypercentageformatted'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ToolLpDataForPlanPageResponse from a dict"""
        if obj is None:
            return None

        relaxed = settings.relaxe_all_models or "ToolLpDataForPlanPageResponse" in settings.relaxed_models

        if not isinstance(obj, dict):
            return cls.model_construct(**obj) if relaxed else cls.model_validate(obj)

        data = {
            "competencies": [ToolLpDataForPlanPageResponseCompetenciesInner.from_dict(_item) for _item in obj["competencies"]] if obj.get("competencies") is not None else None,
            "competencycount": obj.get("competencycount"),
            "contextid": obj.get("contextid"),
            "plan": CoreCompetencyListUserPlansResponseInner.from_dict(obj["plan"]) if obj.get("plan") is not None else None,
            "pluginbaseurl": obj.get("pluginbaseurl"),
            "proficientcompetencycount": obj.get("proficientcompetencycount"),
            "proficientcompetencypercentage": obj.get("proficientcompetencypercentage"),
            "proficientcompetencypercentageformatted": obj.get("proficientcompetencypercentageformatted")
        }
        _obj = cls.model_construct(**data) if relaxed else cls.model_validate(data)
        return _obj


